/**
 * @description true for pass the validation(valid), false for failed to pass (not valid)
 */
import { Observable } from 'rxjs';
export interface ValidationFn {
    (value: any): boolean | null;
}
export interface AsyncValidationFn {
    (value: any): Observable<boolean | null>;
}
export declare class ValidationFnFactory {
    /**
     * Validator that requires controls to have a value greater than a number.
     */
    static min(min: number): ValidationFn;
    /**
     * Validator that requires controls to have a value less than a number.
     */
    static max(max: number): ValidationFn;
    /**
     * Validator that requires controls to have a non-empty value.
     */
    static required(): ValidationFn;
    /**
     * Validator that performs email validation.
     */
    static email(value: any): boolean | null;
    /**
     * Validator that requires controls to have a value of a minimum length.
     */
    static minLength(minLength: number): ValidationFn;
    /**
     * Validator that requires controls to have a value of a maximum length.
     */
    static maxLength(maxLength: number): ValidationFn;
    /**
     * Validator that requires a control to match a regex to its value.
     */
    static pattern(pattern: string | RegExp): ValidationFn;
    /**
     * No-op validator.
     */
    static nullValidator(value: any): boolean | null;
}
