var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, EventEmitter, Input, Output, ViewChildren } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { clone } from 'lodash-es';
/**
 * This component is the actual visible columns, can be drag and reorder by user reaction.
 */
let OcTableHeaderComponent = class OcTableHeaderComponent {
    constructor(translateS) {
        this.translateS = translateS;
        this.resizeEnd = new EventEmitter();
        this.reorderEnd = new EventEmitter();
        this.resortEnd = new EventEmitter();
        // global status of this comp
        this.isDraggingDivider = false;
        this.insertBeforeIndex = -1;
    }
    toggleSort(column) {
        // toggle sortOrder
        let order;
        if (!column.sortOrder || column.sortOrder === 'desc') {
            order = 'asc';
        }
        else {
            order = 'desc';
        }
        column.datasource.setSortConf(column.value, order);
        if (column.datasource.rows.length > 0) {
            column.datasource
                .sort(column.value, order)
                .subscribe((source) => {
            });
        }
        this.resortEnd.emit({
            column: column,
            sortOrder: order,
        });
    }
    ngOnChanges(changes) {
        // array change can only be detected by InterableDiffers, which is used by *ngFor. So this array need to be immutable here
        if (changes['initialedColumnWidth'] && changes['initialedColumnWidth'].currentValue) {
            this.internalColumnWidth = clone(changes['initialedColumnWidth'].currentValue);
            // console.log('changes', this.internalColumnWidth);
        }
    }
    onDividerDragging({ event, model, distance }, i) {
        this.isDraggingDivider = true;
        this.internalColumnWidth[i] = this.initialedColumnWidth[i] + distance;
        // console.log(this.tableHeader.nativeElement.width);
        // console.log(distance, this.initialedColumnWidth[i], this.internalColumnWidth[i]);
    }
    onDividerDragEnd({ model, distance }, i) {
        this.isDraggingDivider = false;
        this.resizeEnd.emit({
            column: model,
            width: this.internalColumnWidth[i],
        });
    }
    onPlaceholderDragStart({ event, element, model, distance }, i) {
        // element.nativeElement.style.background = '#FF9800';
        element.nativeElement.style['z-index'] = 5;
        // prepare initial state of relativeDistance
        let orderedColumnArray = this.columns;
        this.relativeDistanceArray = orderedColumnArray.map((col, i) => {
            let colBox = this.placeholderRef.toArray()[i].nativeElement.getBoundingClientRect();
            let refBox = element.nativeElement.getBoundingClientRect();
            let colPositionX = colBox.left + colBox.width / 2;
            let referencePositionX = refBox.left + refBox.width / 2;
            return colPositionX - referencePositionX;
        });
    }
    onPlaceholderDragging({ event, element, model, distance }, i) {
        // move first
        element.nativeElement.style.left = distance + 'px';
        // judge if need to reorder
        this.insertBeforeIndex = this.calculateInsertBeforeIndex(distance, i);
        // console.log(this.insertBeforeIndex);
        // console.log(distance)
        // console.log(this.relativeDistanceArray);
        // console.log(insertBeforeIndex);
    }
    onPlaceholderDragEnd({ event, element, model, distance }, i) {
        // reOrder
        let newOrder = this.columns.map(col => col.value);
        if (this.insertBeforeIndex !== -1) {
            // save, set to null,
            let toBeInsert = newOrder[i];
            newOrder[i] = null;
            // insert
            newOrder.splice(this.insertBeforeIndex, 0, toBeInsert);
            // delete null
            let nullIndex = newOrder.findIndex(x => x === null);
            newOrder.splice(nullIndex, 1);
            // console.log(this.insertBeforeIndex);
            // console.log(newOrder);
        }
        // clear style
        element.nativeElement.style.left = 0;
        element.nativeElement.style.background = 'white';
        element.nativeElement.style['z-index'] = 2;
        // emit
        this.reorderEnd.emit({
            order: newOrder
        });
    }
    calculateInsertBeforeIndex(distance, i) {
        // find index
        let insertBeforeIndex; // number
        if (distance < 0) {
            for (let i = 0; i < this.relativeDistanceArray.length; i++) {
                if (this.relativeDistanceArray[i] < 0 && this.relativeDistanceArray[i] > distance) {
                    insertBeforeIndex = i;
                    break;
                }
                insertBeforeIndex = -1;
            }
        }
        else {
            for (let i = 0; i < this.relativeDistanceArray.length; i++) {
                if (this.relativeDistanceArray[i] > 0 && this.relativeDistanceArray[i] > distance) {
                    insertBeforeIndex = i;
                    break;
                }
                // last cell if exceed max length
                if (this.relativeDistanceArray[i] > 0)
                    insertBeforeIndex = i + 1;
            }
        }
        // // if equal to self, reset to 1
        // if (insertBeforeIndex === i) {
        //   insertBeforeIndex = -1;
        // }
        return insertBeforeIndex;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Array)
], OcTableHeaderComponent.prototype, "columns", void 0);
__decorate([
    Input(),
    __metadata("design:type", Array)
], OcTableHeaderComponent.prototype, "initialedColumnWidth", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], OcTableHeaderComponent.prototype, "resizeEnd", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], OcTableHeaderComponent.prototype, "reorderEnd", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], OcTableHeaderComponent.prototype, "resortEnd", void 0);
__decorate([
    ViewChildren('placeholderRef'),
    __metadata("design:type", Object)
], OcTableHeaderComponent.prototype, "placeholderRef", void 0);
OcTableHeaderComponent = __decorate([
    Component({
        selector: 'oc-table-header',
        template: `
    <div class = "table-header"
         [ngClass] = "{draggingDivider: isDraggingDivider, hover: isHover}"
         *ngIf = "!!initialedColumnWidth && !!columns">
      <ng-template ngFor let-column let-i = "index" [ngForOf] = "columns">
        <span class = "column-placeholder"
              #placeholderRef
              [ngStyle] = "{ 'width.px': i !== columns?.length - 1 ? internalColumnWidth[i] + 21 : internalColumnWidth[i] + 45}"
              mat-ripple
              draggable
              [model] = "column"
              [ngClass] = "{dragging: column.dragging}"
              (dragStart) = "onPlaceholderDragStart($event, i)"
              (dragging) = "onPlaceholderDragging($event, i)"
              (dragEnd) = "onPlaceholderDragEnd($event, i)">
          <span class = "column-name translated" 
                *ngIf = "column.datasource && column.datasource.translatePipePrefix && !!translateS && !!translateS.currentLang">
            {{column.datasource.translatePipePrefix + '.' + column.value | translate}}</span>
          <span class = "column-name" *ngIf = "column.datasource && (!translateS || !translateS?.currentLang)">{{column.value}}</span>
          <i class = "material-icons sort-icon"
             *ngIf = "column.isSortable"
             [class.sort-asc] = "column.sortOrder === 'asc'"
             [class.sort-desc] = "column.sortOrder === 'desc'"
             (click) = "toggleSort(column)">
             arrow_upward
          </i>
        </span>
        <span *ngIf = "i !== columns?.length - 1"
              draggable
              class = "draggable divider"
              [ngClass] = "{dragging: column.dragging}"
              [model] = "column"
              (dragging) = "onDividerDragging($event, i)"
              (dragEnd) = "onDividerDragEnd($event, i)"></span>
      </ng-template>
    </div>
  `,
        styles: [`:host {
  display: block;
  position: relative; }

div.table-header {
  display: block;
  white-space: nowrap;
  overflow: hidden;
  z-index: 2;
  background: transparent;
  cursor: pointer; }
  div.table-header.draggingDivider span.column-placeholder {
    cursor: col-resize; }
  div.table-header:not(.draggingDivider) span.column-placeholder.dragging {
    animation: shake-horizontal 0.6s cubic-bezier(0.455, 0.03, 0.515, 0.955) both; }

@keyframes shake-horizontal {
  0%,
  100% {
    transform: translateX(0); }
  10%,
  30%,
  50%,
  70% {
    transform: translateX(-3px); }
  20%,
  40%,
  60% {
    transform: translateX(3px); }
  80% {
    transform: translateX(2px); }
  90% {
    transform: translateX(-2px); } }
  div.table-header:hover span.draggable.divider {
    border-right: 1px solid rgba(0, 0, 0, 0.12); }

span.draggable.divider {
  display: inline-block;
  z-index: 2;
  width: 3px;
  height: 30px;
  position: relative;
  top: 13px;
  vertical-align: top;
  border-right: 1px solid white;
  cursor: col-resize !important;
  background: white;
  user-select: none; }
  span.draggable.divider::selection {
    background: transparent !important; }
  span.draggable.divider:hover {
    border-right: 3px solid rgba(0, 0, 0, 0.12) !important;
    background: white; }
  span.draggable.divider.dragging {
    border-right: 3px solid rgba(0, 0, 0, 0.12) !important;
    background: white; }

span.column-placeholder {
  display: inline-block;
  height: 55px;
  position: relative;
  z-index: 2;
  vertical-align: bottom;
  background: white;
  cursor: pointer;
  user-select: none; }
  span.column-placeholder .sort-icon {
    color: rgba(0, 0, 0, 0.26);
    opacity: 0;
    transition: transform 0.25s, opacity 0.25s;
    vertical-align: middle;
    z-index: 4; }
  span.column-placeholder .sort-asc {
    transform: rotate(0deg);
    opacity: 1;
    color: rgba(0, 0, 0, 0.87); }
  span.column-placeholder .sort-desc {
    transform: rotate(180deg);
    opacity: 1;
    color: rgba(0, 0, 0, 0.87); }
  span.column-placeholder:hover .sort-icon {
    opacity: 1; }
  span.column-placeholder span.column-name {
    font-size: 12px;
    line-height: 55px;
    padding-left: 24px;
    font-weight: bold;
    white-space: nowrap;
    vertical-align: middle;
    overflow-x: hidden;
    text-align: left;
    color: rgba(0, 0, 0, 0.54);
    user-select: none; }
`]
    }),
    __metadata("design:paramtypes", [TranslateService])
], OcTableHeaderComponent);
export { OcTableHeaderComponent };
