var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input } from '@angular/core';
import { OcTableDataSource } from '../datasource/table-datasource.model';
import { TranslateService } from '@ngx-translate/core';
import { OcTableConfigStoreService } from '../services/config-store.service';
let OcTablePaginatorComponent = class OcTablePaginatorComponent {
    constructor(translateS) {
        this.translateS = translateS;
        this.perPage = 10;
    }
    ngOnChanges(changes) {
        // this.updatePages();
        if (changes['datasource'] && changes['datasource'].currentValue) {
            if (this.subscription)
                this.subscription.unsubscribe();
            this.subscription = this.datasource.pagerStateChangeInternal.subscribe((x) => {
                this.updatePages();
            });
        }
    }
    perPageOnSelect(evt) {
        this.perPage = parseInt(evt.value);
        this.paginate(1);
    }
    updatePages() {
        this.totalPages = Math.floor(this.datasource.getPagerConf()['totalRows'] / this.perPage) + 1;
        if (this.datasource.getPagerConf()['perPage']) {
            this.perPage = this.datasource.getPagerConf()['perPage'];
        }
        else {
            this.perPage = 10;
        }
        if (this.datasource.getPagerConf()['curPage']) {
            this.curPage = this.datasource.getPagerConf()['curPage'];
        }
    }
    paginate(page) {
        if (this.datasource.rows.length > 0) {
            this.datasource
                .paginate(page, this.perPage)
                .subscribe(source => {
                this.curPage = page;
                this.updatePages();
                this.saveConfig();
            });
        }
    }
    saveConfig() {
        const rtConfigStore = new OcTableConfigStoreService();
        rtConfigStore.saveConfigByDataSource(this.datasource);
    }
    ngOnDestroy() {
        if (this.subscription)
            this.subscription.unsubscribe();
    }
};
__decorate([
    Input(),
    __metadata("design:type", Number)
], OcTablePaginatorComponent.prototype, "perPage", void 0);
__decorate([
    Input(),
    __metadata("design:type", OcTableDataSource)
], OcTablePaginatorComponent.prototype, "datasource", void 0);
OcTablePaginatorComponent = __decorate([
    Component({
        selector: 'oc-table-paginator',
        template: `
    <div class = "container" *ngIf = "datasource?.rows?.length>0">
      <div class = "hint-container">
        <span *ngIf = "!!translateS && !!translateS.currentLang" >{{'table.perPage' | translate}}</span>
        <span *ngIf = "!translateS || !translateS.currentLang" >{{'Rows per page' | translate}}</span>
      </div>

      <div class = "page-picker">
        <mat-select [ngModel] = "perPage.toString()" (selectionChange) = "perPageOnSelect($event)">
          <mat-option value = "10">10</mat-option>
          <mat-option value = "20">20</mat-option>
          <mat-option value = "50">50</mat-option>
        </mat-select>
      </div>

      <div class = "status-container">
    <span>{{(curPage-1) * perPage + 1}} -
          {{curPage * perPage > datasource.getPagerConf()['totalRows'] ? datasource.getPagerConf()['totalRows'] : curPage * perPage}} of
          {{datasource.getPagerConf()['totalRows']}}
    </span>
      </div>

      <div class = "button-container" *ngIf = "datasource?.rows?.length && datasource?.columns?.length">

        <button mat-icon-button [disabled] = "curPage == 1" (click) = "paginate(curPage - 1)">
          <i class = "material-icons">keyboard_arrow_left</i>
        </button>

        <button mat-icon-button [disabled] = "curPage * perPage >= datasource.getPagerConf()['totalRows']"
                (click) = "paginate(curPage + 1)">
          <i class = "material-icons">keyboard_arrow_right</i>
        </button>
      </div>

    </div>
  `,
        styles: [`:host {
  display: block; }
  :host .container {
    display: flex;
    flex-direction: row;
    justify-content: flex-end;
    align-items: center;
    padding: 8px 0; }
    :host .container > * {
      display: inline-block; }

.hint-container {
  margin-right: 32px;
  font-size: 12px;
  color: rgba(0, 0, 0, 0.54); }

.status-container {
  margin-right: 24px;
  font-size: 12px;
  color: rgba(0, 0, 0, 0.54); }

.button-container {
  margin-right: 6px;
  padding-bottom: 3px; }
  .button-container:first-child {
    margin-right: 8px; }

.page-picker {
  width: 40px;
  margin-right: 32px; }

:host /deep/ .mat-select-trigger {
  font-size: 12px;
  height: inherit;
  min-width: 40px; }

:host /deep/ .mat-select-value-text {
  line-height: inherit;
  color: rgba(0, 0, 0, 0.54); }

:host /deep/ .mat-form-field-underline {
  height: 0; }
`]
    }),
    __metadata("design:paramtypes", [TranslateService])
], OcTablePaginatorComponent);
export { OcTablePaginatorComponent };
