import { Observable, ReplaySubject } from 'rxjs';
import { OcTableColumn } from './oc-table-column.model';
import { OcTableRow } from './oc-table-row.model';
import { OcTableConfig, OcTableDropdownItem, OcTableFloatingButton } from '../oc-table.interface';
/**
 * The internal dataSource Object for table
 * Avoid use this directly by 2.0 design
 */
export interface SorterState {
    field: string;
    dir: 'asc' | 'desc' | null;
}
export interface PagerState {
    curPage: number;
    perPage: number;
    totalRows?: number;
}
export declare class OcTableDataSource {
    isLoading: boolean;
    pagerStateChangeInternal: ReplaySubject<PagerState>;
    pagerStateChange: Observable<PagerState>;
    sorterStateChangeInternal: ReplaySubject<SorterState>;
    sorterStateChange: Observable<SorterState>;
    rows: OcTableRow[];
    columns: OcTableColumn[];
    dropdown: OcTableDropdownItem[];
    floatingButtons: OcTableFloatingButton[];
    translatePipePrefix: any;
    config: OcTableConfig;
    pagerState: PagerState;
    private sorterState;
    private data;
    constructor(rows: OcTableRow[], columns: OcTableColumn[], config?: OcTableConfig);
    static constructFromData(data: Array<any>): OcTableDataSource;
    static constructFromDataAndConfig(data: Array<any>, config?: OcTableConfig): OcTableDataSource;
    /**
     * Reload DataSource,
     * @param {Array<any>} data
     */
    reloadFromData(data: Array<any>): void;
    /**
     * Accessors
     * @param {string} name
     * @returns {OcTableColumn}
     */
    getColumnByName(name: string): OcTableColumn;
    /**
     * ChangeSource
     */
    /**
     * TableDataOperation Interface
     */
    setSortConf(field: string, dir: 'asc' | 'desc' | null): void;
    getSortConf(): SorterState;
    getPagerConf(): PagerState;
    getSortedColumn(): OcTableColumn;
    setPagerConf(curPage: number, perPage: number, totalRows?: number): PagerState;
    sort(field: string, dir: 'asc' | 'desc' | null, compareFunc?: Function, defaultPage?: number): Observable<OcTableDataSource>;
    paginate(page: number, perPage: number): Observable<OcTableDataSource>;
    /**
     * Other Operation
     */
    reorderColumn(colNameWithOrder: string[]): void;
    /**
     * @Internal
     */
    initFromConfig(config: OcTableConfig): void;
    private createColumnsFromData;
    private createColumnsFromConfig;
    protected renewCellsRef(): void;
    private renewConfigRef;
}
