var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { fromEvent as observableFromEvent } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { Directive, ElementRef, EventEmitter, HostBinding, HostListener, Input, Output } from '@angular/core';
/**
 * Draggable Directive for Angular2
 *
 * Inspiration:
 *   https://github.com/AngularClass/angular2-examples/blob/master/rx-draggable/directives/draggable.ts
 *   http://stackoverflow.com/questions/35662530/how-to-implement-drag-and-drop-in-angular2
 */
let DraggableDirective = class DraggableDirective {
    constructor(element) {
        this.duration = 200;
        this.dragStart = new EventEmitter();
        this.dragging = new EventEmitter();
        this.dragEnd = new EventEmitter();
        this.longPressStart = new EventEmitter();
        this.longPressing = new EventEmitter();
        this.longPressEnd = new EventEmitter();
        this.mouseX = 0;
        this.mouseY = 0;
        this.isDragging = false;
        this.distance = 0;
        this.element = element;
    }
    get press() { return this.pressing; }
    get isLongPress() {
        return this.isLongPressing;
    }
    get draggable() { return true; }
    onMouseDown(event) {
        // don't do right/middle clicks
        if (event.which !== 1)
            return;
        // don't start drag if its on resize handle
        const target = event.target;
        if (target.classList.contains('resize-handle'))
            return;
        this.mouseX = event.clientX;
        this.mouseY = event.clientY;
        this.pressing = true;
        this.isLongPressing = false;
        const mouseup = observableFromEvent(document, 'mouseup');
        this.subscription = mouseup.subscribe((ev) => this.onMouseup());
        this.timeout = setTimeout(() => {
            this.isLongPressing = true;
            this.longPressStart.emit({
                event,
                model: this.model
            });
            this.model.dragging = true;
            this.onDragMousedown(event);
            this.subscription.add(observableFromEvent(document, 'mousemove').pipe(takeUntil(mouseup))
                .subscribe((mouseEvent) => this.onMouseMove(mouseEvent)));
            this.loop(event);
        }, this.duration);
        this.loop(event);
    }
    onMouseMove(event) {
        if (this.pressing && !this.isLongPressing) {
            const xThres = Math.abs(event.clientX - this.mouseX) > 10;
            const yThres = Math.abs(event.clientY - this.mouseY) > 10;
            if (xThres || yThres) {
                this.endPress();
            }
        }
    }
    loop(event) {
        if (this.isLongPressing) {
            this.timeout = setTimeout(() => {
                this.longPressing.emit({
                    event,
                    model: this.model
                });
                this.loop(event);
            }, 50);
        }
    }
    endPress() {
        clearTimeout(this.timeout);
        this.isLongPressing = false;
        this.pressing = false;
        this.longPressEnd.emit({
            model: this.model
        });
        this.dragEventTarget = event;
        setTimeout(() => {
            this.model.dragging = false;
        }, 5);
    }
    onMouseup() {
        this.endPress();
    }
    onDragMouseup(event) {
        if (!this.isDragging)
            return;
        this.isDragging = false;
        if (this.subscription) {
            this._destroySubscription();
            this.dragEnd.emit({
                event,
                element: this.element,
                model: this.model,
                distance: this.distance
            });
        }
    }
    onDragMousedown(event) {
        // we only want to drag the inner header text
        event.preventDefault();
        this.isDragging = true;
        const mouseDownPos = { x: event.clientX };
        const mouseup = observableFromEvent(document, 'mouseup');
        let mouseUpSub = mouseup
            .subscribe((ev) => this.onDragMouseup(ev));
        const mouseMoveSub = observableFromEvent(document, 'mousemove').pipe(takeUntil(mouseup))
            .subscribe((ev) => this.onDragMove(ev, mouseDownPos));
        this.subscription.add(mouseMoveSub);
        this.subscription.add(mouseUpSub);
        this.dragStart.emit({
            event,
            element: this.element,
            model: this.model
        });
    }
    onDragMove(event, mouseDownPos) {
        if (!this.isDragging)
            return;
        const x = event.clientX - mouseDownPos.x;
        // this.element.style.left = `${x}px`;
        // console.log('---');
        // console.log(event.clientX, mouseDownPos.x);
        // console.log(this.element.style.left);
        // console.log('---');
        this.distance = x;
        this.dragging.emit({
            event,
            element: this.element,
            model: this.model,
            distance: this.distance
        });
    }
    ngOnDestroy() {
        this._destroySubscription();
    }
    _destroySubscription() {
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = undefined;
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], DraggableDirective.prototype, "model", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], DraggableDirective.prototype, "duration", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], DraggableDirective.prototype, "dragStart", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], DraggableDirective.prototype, "dragging", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], DraggableDirective.prototype, "dragEnd", void 0);
__decorate([
    HostBinding('class.press'),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], DraggableDirective.prototype, "press", null);
__decorate([
    HostBinding('class.longpress'),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], DraggableDirective.prototype, "isLongPress", null);
__decorate([
    HostBinding('class.draggable'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DraggableDirective.prototype, "draggable", null);
__decorate([
    HostListener('mousedown', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [MouseEvent]),
    __metadata("design:returntype", void 0)
], DraggableDirective.prototype, "onMouseDown", null);
DraggableDirective = __decorate([
    Directive({ selector: '[draggable]' }),
    __metadata("design:paramtypes", [ElementRef])
], DraggableDirective);
export { DraggableDirective };
