import { Observable } from 'rxjs';
import { PagerState, SorterState } from './datasource/table-datasource.model';
import { OcTableRow } from './datasource/oc-table-row.model';
export interface OcPipeTransformFn {
    (value: any): any;
}
export interface AsyncDataFetchFn {
    (pagerState: PagerState, sorterState: SorterState): Observable<{
        data: any;
        totalRows: number;
    }>;
}
export interface OcTableRowConfigFn {
    (row: OcTableRow): Observable<boolean>;
}
export interface OcTableDropdownItem {
    value: string;
    iconName: string;
    clickEventName: string;
}
export interface OcTableFloatingButton {
    value: string;
    iconName: string;
    clickEventName: string;
    visible?: boolean | OcTableRowConfigFn;
}
export interface OcTableColumnConfig {
    name: string;
    type?: string;
    numeric?: boolean;
    pipe?: OcPipeTransformFn;
    width?: number;
    minWidth?: number;
    lineBreak?: boolean;
    sortable?: boolean;
    sortFn?: Function;
    sortOrder?: 'asc' | 'desc' | null;
}
export interface OcTablePagerConfig {
    perPage: number;
    totalItems?: number;
    defaultPage?: number;
}
/**
 * @name: Use to identify different tables, Cache table settings is base on this, Translate prefix is same as name.
 * Example translate json file structure:
 * "members-table": {
    "status": "Status", // column translate
    "roleName": "Role",
    "enabled": "Status",
    "cellValue": {    // cell value translate
      "enabled": {
        "true": "Active",
        "false": "Suspended"
      },
      "status": {
        "true": "Active",
        "false": "Suspended"
      },
      "roleName": {
        "ns_admin": "Namespace Admin",
        "developer": "Developer"
      }
    }
  },
 *
 *
 */
export interface OcTableConfig {
    name: string;
    useSettingCache?: boolean;
    asyncDataFetchFn?: AsyncDataFetchFn;
    columns?: OcTableColumnConfig[];
    dropdown?: OcTableDropdownItem[];
    floatingButtons?: OcTableFloatingButton[];
    pager?: OcTablePagerConfig;
    draggable?: boolean;
    orderable?: boolean;
    canEdit?: boolean | OcTableRowConfigFn;
    canDelete?: boolean | OcTableRowConfigFn;
}
