// singleton
export class OcTableConfigStoreService {
    constructor() {
        if (!!OcTableConfigStoreService.instance) {
            return OcTableConfigStoreService.instance;
        }
        OcTableConfigStoreService.instance = this;
    }
    getSavedConfig(name) {
        if (!this.getLocalStorageObject('tableConfig'))
            this.setLocalStorageObject('tableConfig', {});
        return this.getLocalStorageObject('tableConfig')[name];
    }
    saveConfig(config) {
        if (!config || !config.name) {
            throw new Error('Save table config failed, invalid config');
        }
        if (!this.getLocalStorageObject('tableConfig'))
            this.setLocalStorageObject('tableConfig', {});
        let saveConfig = this.getLocalStorageObject('tableConfig');
        saveConfig[config.name] = config;
        // console.log('tobeSave2', saveConfig);
        this.setLocalStorageObject('tableConfig', saveConfig);
    }
    saveConfigByDataSource(dataSource) {
        if (!dataSource || !dataSource.config || !dataSource.config.name
            || !dataSource.config.columns || !dataSource.config.columns.length) {
            throw new Error('Save table config failed, invalid config');
        }
        let colsRtConfig = dataSource.columns.map(col => {
            let colRtConfig = {};
            colRtConfig.name = col.value;
            if (!!col.width)
                colRtConfig.width = col.width;
            if (!!col.sortOrder)
                colRtConfig.sortOrder = col.sortOrder;
            return colRtConfig;
        });
        let pagerRtConfig = dataSource.getPagerConf();
        let tableRtConfig = {
            name: dataSource.config.name,
            pager: pagerRtConfig,
            columns: colsRtConfig
        };
        // console.log('tobeSave', tableRtConfig);
        return this.saveConfig(tableRtConfig);
    }
    destroyConfig(name) {
        window.localStorage['tableConfig'][name] = null;
    }
    destroyAllConfig() {
        window.localStorage.removeItem('tableConfig');
    }
    setLocalStorageObject(key, value) {
        window.localStorage.setItem(key, JSON.stringify(value));
    }
    getLocalStorageObject(key) {
        let value = window.localStorage.getItem(key);
        return value && JSON.parse(value);
    }
}
OcTableConfigStoreService.instance = null;
