import { OcTableConfigStoreService } from './config-store.service';
/**
 * Singleton that provide config initialize serivice for table,
 * Data: filter --> sort column order --> Map<any,any>
 */
export class OcTableConfigInitializer {
    constructor() {
        if (!!OcTableConfigInitializer.instance) {
            return OcTableConfigInitializer.instance;
        }
        OcTableConfigInitializer.instance = this;
    }
    filterData(data, config) {
        data = this.toMapArray(data);
        if (!config || !config.columns || !config.columns.length)
            return data;
        let filterKey = config.columns.map(x => x.name);
        // pick items by key
        return data.map(col => this.pickValue(col, filterKey));
    }
    bindPipes(dataSource, config) {
        if (!config || !config.columns || !config.columns.length || !dataSource)
            return dataSource;
        dataSource.columns.forEach((col, i) => {
            if (config.columns[i] || config.columns[i].pipe) {
                col.pipe = config.columns[i].pipe;
            }
        });
    }
    validateConfig(data, config) {
        // 1. check columns name exist
    }
    mergeRuntimeConfig(config) {
        if (!config || !config.name) {
            throw new Error('When merging runtime config: invalid config:' + config);
        }
        let rtConfigStore = new OcTableConfigStoreService();
        let rtConfig = rtConfigStore.getSavedConfig(config.name);
        if (!!rtConfig) {
            // merge columns
            let mergedColumns = rtConfig.columns.map(col => {
                let mergedCol = {};
                let originalCol = config.columns.find(x => x.name === col.name);
                if (!!originalCol) {
                    mergedCol = Object.assign(originalCol);
                }
                if (!!col.width) {
                    mergedCol.width = col.width;
                }
                return mergedCol;
            });
            config.columns = mergedColumns;
            // merge pager
            if (!!rtConfig.pager && !!rtConfig.pager.perPage && config.pager) {
                config.pager.perPage = rtConfig.pager.perPage;
            }
        }
        // console.log(config)
        return config;
    }
    toMapArray(data) {
        if (!data || !data.length || data[0] instanceof Map)
            return data;
        // if no config, transform object[] to map[]
        return data.map(obj => {
            let m = new Map();
            for (let k in obj) {
                m.set(k, obj[k]);
            }
            return m;
        });
    }
    pickValue(v, keys) {
        let m = new Map();
        keys.forEach(k => m.set(k, v.get(k)));
        return m;
    }
}
OcTableConfigInitializer.instance = null;
